// AccessEaseDoc.cpp : implementation of the CAccessEaseDoc class
//

#include "stdafx.h"
#include "AllMyIncludes.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAccessEaseDoc

IMPLEMENT_DYNCREATE(CAccessEaseDoc, CDocument)

BEGIN_MESSAGE_MAP(CAccessEaseDoc, CDocument)
	//{{AFX_MSG_MAP(CAccessEaseDoc)
	ON_COMMAND(ID_EDIT_CHANGE_PSWD, OnEditChangePswd)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CHANGE_PSWD, OnUpdateEditChangePswd)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAccessEaseDoc construction/destruction

CAccessEaseDoc::CAccessEaseDoc()
{
	m_lastTimeModified = 0;
}

CAccessEaseDoc::~CAccessEaseDoc()
{
}

BOOL CAccessEaseDoc::OnNewDocument()
{
	P_APP->m_strEncryptionPswd = _T("");
		
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAccessEaseDoc serialization

void CAccessEaseDoc::Serialize(CArchive& ar)
{
	int nSites = 0;
	CString aStr;

	if (ar.IsStoring())
	{
		ar << CString(_strPgmId);

		ar << encode(P_APP->m_strEncryptionPswd, P_APP->m_strEncryptionPswd);

		nSites = m_sitePtrArray.GetSize();		
		ar << nSites;

		for(int i = 0; i < nSites; i++)
		{
			ar << (CSiteInfo *)m_sitePtrArray.GetAt(i);
		}  
	}
	else // retrieving
	{
		ar >> aStr;	// pgm ID, checked in open
		ar >> aStr;	// password, checked in open
		ar >> nSites; // number of sites
		 
		for(int i = 0; i < nSites; i++)
		{
			CSiteInfo * pSite = new CSiteInfo;
			ASSERT_VALID(pSite);

			ar >> pSite;
			m_sitePtrArray.Add(pSite);
		}				
	}
}

/////////////////////////////////////////////////////////////////////////////
// CAccessEaseDoc diagnostics

#ifdef _DEBUG
void CAccessEaseDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CAccessEaseDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}

/////////////////////////////////////////////////////////////////////////////
// CAccessEaseDoc commands

/////////////////////////////////////////////////////////////////////////////
#endif //_DEBUG

void CAccessEaseDoc::DeleteContents() 
{
	int nSites = m_sitePtrArray.GetSize();

	for(int i = 0; i < nSites; i++)
	{
		delete (CSiteInfo * )m_sitePtrArray.GetAt(i);
	}

	m_sitePtrArray.RemoveAll();
	
	CDocument::DeleteContents();
}

void CAccessEaseDoc::DeleteSite(CSiteInfo * pSite)
{
	int nSites = m_sitePtrArray.GetSize();

	for(int i = 0; i < nSites; i++)
	{
		if((CSiteInfo *)m_sitePtrArray[i] == pSite)
		{
			m_sitePtrArray.RemoveAt(i);
			delete pSite;
			break;
		}
	}
}

void CAccessEaseDoc::SetModifiedFlag(BOOL bModified)
{
	if(bModified && !IsModified())
	{
		SetTitle(GetTitle() + _strIndicator);
	}

	if(IsModified() && !bModified) // remove "*" to view's title bar
	{
		CString strTitle = GetTitle();
		SetTitle(strTitle.Left(strTitle.GetLength() - strlen(_strIndicator)));
	}

	UpdateFrameCounts();
	CDocument::SetModifiedFlag(bModified);
}

void CAccessEaseDoc::OnEditChangePswd() 
{
	CPswdDlg aDlg;
	// get file name for password
	aDlg.m_strPswd = P_APP->m_strEncryptionPswd;
	aDlg.m_strFileName = GetTitle();
	aDlg.m_strFileName = _T("File:  ") + aDlg.m_strFileName;

	// remove '*' in modified file name
	if(aDlg.m_strFileName[aDlg.m_strFileName.GetLength() - 1] == _T('*'))
	{
		aDlg.m_strFileName.SetAt(aDlg.m_strFileName.GetLength() - 1, _T(' '));
	}			

	if(IDOK == aDlg.DoModal() && P_APP->m_strEncryptionPswd != aDlg.m_strPswd)
	{
		aDlg.m_strPswd.TrimLeft();
		aDlg.m_strPswd.TrimRight();
		P_APP->m_strEncryptionPswd = aDlg.m_strPswd;
		SetModifiedFlag(TRUE);
	}
}

void CAccessEaseDoc::OnUpdateEditChangePswd(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_sitePtrArray.GetSize() > 0);	
}

// open, check for file type, get password, check password, close file, and then reopen for serialized input 
BOOL CAccessEaseDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	CFile aFile;
	CFileException ex;

	if(aFile.Open(lpszPathName, CFile::modeRead | CFile::shareDenyWrite, &ex))
	{
		CArchive ar(&aFile, CArchive::load);     
		CString aStr;
		int retVal = IDCANCEL;

		ar >> aStr; // file type string
			
		if(aStr == _strPgmId) // is this an AccessEase file?
		{
			ar >> aStr; // get encrtpted password

			if(!aStr.IsEmpty()) // get password from user
			{
				CPswdDlg aDlg;
				// get file name to display in dialog
				aDlg.m_strFileName = ar.GetFile()->GetFilePath();
				aDlg.m_strFileName = aDlg.m_strFileName.Mid(aDlg.m_strFileName.ReverseFind(_T('\\')) + 1);
				aDlg.m_strFileName = _T("File:  ") + aDlg.m_strFileName;

				while(IDOK == (retVal = aDlg.DoModal()))
				{
					if(aDlg.m_strPswd == decode(aDlg.m_strPswd, aStr))
					{
						P_APP->m_strEncryptionPswd = aDlg.m_strPswd; // set encryptoon pswd
						break;
					}
					else
					{
						AfxMessageBox(IDS_MSG_BAD_ENCRYPTION_PSWD, MB_ICONEXCLAMATION);
					}
				}
			}
			else // password is null
			{
				retVal = IDOK;
				P_APP->m_strEncryptionPswd = _T("");
			}
		}
		else
		{
			retVal = IDCANCEL;
			AfxMessageBox(IDS_MSG_NOT_ACCESS_EASE_FILE, MB_ICONEXCLAMATION);
		}

		ar.Close();
		aFile.Close();

		// Open and erialize file data
		if(retVal == IDOK && CDocument::OnOpenDocument(lpszPathName))
		{
 			WIN32_FILE_ATTRIBUTE_DATA wfad;
			// save time stamp to compare whem saving file
			if(GetFileAttributesEx(lpszPathName, GetFileExInfoStandard, &wfad))
			{
				m_lastTimeModified = FileTimeToTimet(&wfad.ftLastWriteTime);
			}

			return TRUE;
		}
	}
	else
	{
	  TCHAR szError[1024];
	  ex.GetErrorMessage(szError, sizeof(szError));
	  AfxMessageBox(szError, MB_ICONSTOP);    
	}

	return FALSE;
}

BOOL CAccessEaseDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
 	WIN32_FILE_ATTRIBUTE_DATA wfad;

	if(GetFileAttributesEx(lpszPathName, GetFileExInfoStandard, &wfad))
	{ 
		if(m_lastTimeModified != FileTimeToTimet(&wfad.ftLastWriteTime))
		{
			if(IDYES != AfxMessageBox(IDS_FILE_CHANGED, MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2))
			{
				return FALSE;
			}
		}
	}

	if(CDocument::OnSaveDocument(lpszPathName))
	{
		if(GetFileAttributesEx(lpszPathName, GetFileExInfoStandard, &wfad))
		{
			m_lastTimeModified = FileTimeToTimet(&wfad.ftLastWriteTime);
		}

		return TRUE;
	}

	return FALSE;
}

void CAccessEaseDoc::OnFileSaveAs() 
{
	CDocument::OnFileSaveAs();

 	WIN32_FILE_ATTRIBUTE_DATA wfad;

	if(GetFileAttributesEx(GetPathName(), GetFileExInfoStandard, &wfad))
	{
		m_lastTimeModified = FileTimeToTimet(&wfad.ftLastWriteTime);
	}
}
